// ------------------------------------------------
// File : servmgr.cpp
// Date: 4-apr-2002
// Author: giles
// Desc: 
//		Management class for handling multiple servent connections.
//
// (c) 2002 peercast.org
// ------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------

#include <stdlib.h>
#include "servent.h"
#include "servmgr.h"
#include "inifile.h"
#include "stats.h"

static int debugtest=0;
extern String iniFileName;



// -----------------------------------
ServMgr::ServMgr()
{
	int i;
	for(i=0; i<MAX_SERVENTS; i++)
		servents[i].init();

	authType = AUTH_COOKIE;
	cookieList.init();

	startTime = sys->getTime();

	allowServer1 = Servent::ALLOW_ALL;
	allowServer2 = Servent::ALLOW_BROADCAST;

	clearHostCache();
	password[0]=0;

	allowGnutella = false;

	maxBitrate = 0;
	maxIncoming = MAX_INCOMING;
	maxOutgoing = MAX_OUTGOING;
	maxTryout = MAX_TRYOUT;
	minConnected = MIN_CONNECTED;
	maxStreams = 0;
	refreshHTML = 5;

	tryoutDelay = 10;
	relayBroadcast = 300;
	serventBandwidth = 1000;

	numVersions = 0;

	sessionID.generate();

	isRoot = false;

	forceIP[0]	= 0;		// force server IP

	strcpy(connectHost,"connect1.peercast.org");

	serverHost.fromStr("127.0.0.1:7144");
	network.set("peercast");

	firewalled = FW_UNKNOWN;		
	allowDirect = true;
	autoConnect = true;
	forceLookup = true;
	autoServe = true;
	forceNormal = false;

	replyIDs.init(500);	// 500 active packets

	totalStreams = 0;
	firewallTimeout = 30;
	pauseLog = false;
	showLog = 0;

	shutdownTimer = 0;

	downloadURL[0] = 0;
	rootMsg.clear();


	restartServer=false;

	setFilterDefaults();

}

// -----------------------------------
void	ServMgr::addVersion(unsigned int ver)
{
	for(int i=0; i<numVersions; i++)
		if (clientVersions[i] == ver)
		{
			clientCounts[i]++;
			return;
		}

	if (numVersions < MAX_VERSIONS)
	{
		clientVersions[numVersions] = ver;
		clientCounts[numVersions] = 1;
		numVersions++;
	}
}

// -----------------------------------
void ServMgr::setFilterDefaults()
{
	numFilters = 0;

	filters[numFilters].host.fromStr("255.255.255.255");
	filters[numFilters].flags = ServFilter::F_ALLOW;
	numFilters++;

	filters[numFilters].host.fromStr("127.0.0.1");
	filters[numFilters].flags = ServFilter::F_PRIVATE|ServFilter::F_ALLOW;
	numFilters++;

}
// -----------------------------------
void	ServMgr::setPassiveSearch(unsigned int t)
{
//	if ((t > 0) && (t < 60))
//		t = 60;
//	passiveSearch = t;
}
// -----------------------------------
void ServMgr::addHost(Host &h, bool force)
{
	if (!h.isValid() || h.loopbackIP())
		return;

	if (!force)
		for(int i=0; i<MAX_HOSTCACHE; i++)
			if (hostCache[i].ip == h.ip)
				if (hostCache[i].port == h.port)
					return;

	char str[64];
	h.toStr(str);
	LOG_NETWORK("New host: %s",str);

	h.value = 0;	// make sure dead count is zero
	hostCache[writeHostPtr] = h;
	readHostPtr = writeHostPtr;		// set read ptr to get this host next
	writeHostPtr = (writeHostPtr+1)%MAX_HOSTCACHE;

}
// -----------------------------------
void ServMgr::deadHost(Host &h)
{
	for(int i=0; i<MAX_HOSTCACHE; i++)
		if (hostCache[i].ip == h.ip)
			if (hostCache[i].port == h.port)
				hostCache[i].init();
}
// -----------------------------------
void ServMgr::clearHostCache()
{
	memset(hostCache,0,MAX_HOSTCACHE*sizeof(Host));
	readHostPtr=writeHostPtr=0;
}
	
// -----------------------------------
unsigned int ServMgr::numHosts()
{
	unsigned int cnt = 0;
	for(int i=0; i<MAX_HOSTCACHE; i++)
		if (hostCache[i].ip != 0)
			if (hostCache[i].port != 0)
				cnt++;
	return cnt;
}
// -----------------------------------
Host ServMgr::getHost()
{
	Host h(0,0);
	
	for(int i=0; i<MAX_HOSTCACHE; i++)
	{
		h = hostCache[readHostPtr];
		readHostPtr = (readHostPtr-1)%MAX_HOSTCACHE;
		if (h.ip != 0)
			break;
	}

	return h;
}
// -----------------------------------
int ServMgr::getNewestHosts(Host *hl,int max,Host &rh)
{
	int p = (writeHostPtr > 0) ? writeHostPtr-1 : MAX_HOSTCACHE-1;
	int cnt=0;
	for(int i=0; i<MAX_HOSTCACHE; i++)
	{
		Host *h = &hostCache[p];
		if (!((rh.globalIP() && !h->globalIP()) || rh.isSame(*h) || (h->ip==0)))
		{
			hl[cnt++] = *h;

			if (cnt >= max)
				break;
		}
		if (p>0)
			p--;
		else
			p = (writeHostPtr > 0) ? writeHostPtr-1 : MAX_HOSTCACHE-1;
	}
	return cnt;
}

// -----------------------------------
Servent *ServMgr::findServent(unsigned int ip, unsigned short port)
{
	lock.on();
	for(int i=0; i<MAX_SERVENTS; i++)
	{
		Servent *s = &servents[i];
		if (s->type != Servent::T_NONE)
		{
			Host h = s->getHost();
			if ((h.ip == ip) && (h.port == port))
			{
				lock.off();
				return s;
			}
		}
	}
	lock.off();
	return NULL;

}

// -----------------------------------
Servent *ServMgr::findServent(Servent::TYPE t)
{
	for(int i=0; i<MAX_SERVENTS; i++)
	{
		Servent *s = &servents[i];
		if (s->type == t)
			return s;
	}
	return NULL;
}
// -----------------------------------
Servent *ServMgr::allocServent()
{
	lock.on();
	Servent *s = findServent(Servent::T_NONE);
	if (s)
		s->type = Servent::T_ALLOCATED;

	lock.off();

	if (!s) 
		return NULL;

	return s;
}
// -----------------------------------
unsigned int ServMgr::numConnected(int type)
{
	unsigned int cnt=0;

	for(int i=0; i<MAX_SERVENTS; i++)
	{
		Servent *s = &servents[i];
		if (s->status == Servent::S_CONNECTED)
			if (s->type == type)
				cnt++;
	}
	return cnt;
}
// -----------------------------------
unsigned int ServMgr::numUsed(int type)
{
	unsigned int cnt=0;

	for(int i=0; i<MAX_SERVENTS; i++)
	{
		Servent *s = &servents[i];
		if (s->type == type)
			cnt++;
	}
	return cnt;
}

// -----------------------------------
unsigned int ServMgr::numStreams(bool all)
{
	unsigned int cnt=0;

	for(int i=0; i<MAX_SERVENTS; i++)
	{
		Servent *s = &servents[i];
		if (s->status == Servent::S_CONNECTED)
			if (s->type == Servent::T_STREAM)
				if (all || !s->isPrivate())
					cnt++;
	}
	return cnt;
}
// -----------------------------------
unsigned int ServMgr::totalOutput(bool all)
{
	unsigned int tot = 0;
	for(int i=0; i<MAX_SERVENTS; i++)
	{
		Servent *s = &servents[i];
		if (s->status == Servent::S_CONNECTED)
			if (s->type == Servent::T_STREAM)
				if (all || !s->isPrivate())
					if (s->sock)
						tot += s->sock->bytesOutPerSec;
	}

	return tot;
}

// -----------------------------------
unsigned int ServMgr::numOutgoing()
{
	int cnt=0;

	for(int i=0; i<MAX_SERVENTS; i++)
	{
		Servent *s = &servents[i];
		if ((s->type == Servent::T_INCOMING) ||
		    (s->type == Servent::T_OUTGOING)) 
			cnt++;
	}
	return cnt;
}

// -----------------------------------
bool ServMgr::seenPacket(GnuPacket &p)
{
	for(int i=0; i<MAX_SERVENTS; i++)
	{
		Servent *s = &servents[i];
		if (s->status == Servent::S_CONNECTED)
			if (s->seenIDs.contains(p.id))
				return true;
	}
	return false;
}

// -----------------------------------
int ServMgr::broadcast(GnuPacket &pack,Servent *src)
{
	int cnt=0;
	if (pack.ttl)
	{
		for(int i=0; i<MAX_SERVENTS; i++)
		{
			Servent *s = &servents[i];
			if (s != src)
				if (s->type != Servent::T_STREAM)
					if (s->status == Servent::S_CONNECTED)
						if (!s->seenIDs.contains(pack.id))
							if (s->outputPacket(pack))
								cnt++;
		}
	}

	if (cnt)
		LOG_NETWORK("broadcast: %d (%d) to %d servents",pack.func,pack.ttl,cnt);


	return cnt;
}
// -----------------------------------
int ServMgr::route(GnuPacket &pack, GnuID &routeID, Servent *src)
{
	int cnt=0;
	if (pack.ttl)
	{
		for(int i=0; i<MAX_SERVENTS; i++)
		{
			Servent *s = &servents[i];
			if (s != src)
				if (s->type != Servent::T_STREAM)
					if (s->status == Servent::S_CONNECTED)
						if (s->seenIDs.containsID(routeID))
							if (s->outputPacket(pack))
								cnt++;
		}
	}

	if (cnt)
		LOG_NETWORK("route: %d (%d) to %d servents",pack.func,pack.ttl,cnt);
	return cnt;
}
// -----------------------------------
void ServMgr::setFirewall(FW_STATE state)
{
	if (firewalled != state)
	{
		char *str;
		switch (state)
		{
			case FW_ON:
				str = "ON";
				break;
			case FW_OFF:
				str = "OFF";
				break;
			case FW_UNKNOWN:
			default:
				str = "UNKNOWN";
				break;
		}

		LOG_DEBUG("Firewall is set to %s",str);
		firewalled = state;
	}
}
// -----------------------------------
bool ServMgr::isFiltered(int fl, Host &h)
{
	for(int i=0; i<numFilters; i++)
		if (filters[i].flags & fl)
			if (h.isMemberOf(filters[i].host))
				return true;

	return false;
}

#if 0
// -----------------------------------
bool ServMgr::canServeHost(Host &h)
{
	if (server)
	{
		Host sh = server->getHost();

		if (sh.globalIP() || (sh.localIP() && h.localIP()))
			return true;		
	}
	return false;
}
#endif

// --------------------------------------------------
void writeServerSettings(IniFile &iniFile, unsigned int a)
{
	iniFile.writeBoolValue("allowHTML",a & Servent::ALLOW_HTML);
	iniFile.writeBoolValue("allowData",a & Servent::ALLOW_DATA);
	iniFile.writeBoolValue("allowServent",a & Servent::ALLOW_SERVENT);
	iniFile.writeBoolValue("allowBroadcast",a & Servent::ALLOW_BROADCAST);
}
// --------------------------------------------------
void writeFilterSettings(IniFile &iniFile, ServFilter &f)
{
	char ipstr[64];
	f.host.IPtoStr(ipstr);
	iniFile.writeStrValue("ip",ipstr);
	iniFile.writeBoolValue("private",f.flags & ServFilter::F_PRIVATE);
	iniFile.writeBoolValue("ban",f.flags & ServFilter::F_BAN);
	iniFile.writeBoolValue("allow",f.flags & ServFilter::F_ALLOW);
}
// --------------------------------------------------
void ServMgr::saveSettings(const char *fn)
{
	IniFile iniFile;
	if (!iniFile.openWriteReplace(fn))
	{
		LOG_ERROR("Unable to open ini file");
	}else{

		iniFile.writeSection("Server");
		iniFile.writeIntValue("serverPort",servMgr->serverHost.port);
		iniFile.writeBoolValue("autoServe",servMgr->autoServe);
		iniFile.writeStrValue("icyPassword",servMgr->password);
		iniFile.writeStrValue("forceIP",servMgr->forceIP);
		iniFile.writeBoolValue("isRoot",servMgr->isRoot);
		iniFile.writeIntValue("maxBitrate",servMgr->maxBitrate);
		iniFile.writeIntValue("maxStreams",servMgr->maxStreams);
		iniFile.writeIntValue("maxStreamsPerChannel",chanMgr->maxStreamsPerChannel);
		iniFile.writeIntValue("maxIncoming",servMgr->maxIncoming);
		iniFile.writeIntValue("maxOutgoing",servMgr->maxOutgoing);
		iniFile.writeIntValue("maxTryout",servMgr->maxTryout);
		iniFile.writeIntValue("minConnected",servMgr->minConnected);
		iniFile.writeIntValue("serventBandwidth",servMgr->serventBandwidth);
		iniFile.writeIntValue("firewallTimeout",firewallTimeout);
		iniFile.writeBoolValue("forceNormal",forceNormal);
		iniFile.writeStrValue("rootMsg",rootMsg.cstr());
		iniFile.writeStrValue("authType",servMgr->authType==ServMgr::AUTH_COOKIE?"cookie":"http-basic");
		iniFile.writeStrValue("cookiesExpire",servMgr->cookieList.neverExpire==true?"never":"session");
		iniFile.writeStrValue("network",servMgr->network.cstr());

		iniFile.writeSection("Client");
		iniFile.writeBoolValue("autoConnect",servMgr->autoConnect);
		iniFile.writeStrValue("lookupHost",servMgr->connectHost);
		iniFile.writeIntValue("tryoutDelay",servMgr->tryoutDelay);
		iniFile.writeIntValue("refreshHTML",refreshHTML);
		iniFile.writeIntValue("relayBroadcast",servMgr->relayBroadcast);

		
		iniFile.writeSection("Notify");
			iniFile.writeBoolValue("PeerCast",sys->popupNotifyMask&Sys::NT_PEERCAST);
			iniFile.writeBoolValue("Broadcasters",sys->popupNotifyMask&Sys::NT_BROADCASTERS);
			iniFile.writeBoolValue("TrackInfo",sys->popupNotifyMask&Sys::NT_TRACKINFO);
		iniFile.writeLine("[End]");

		int i;

		for(i=0; i<servMgr->numFilters; i++)
		{
			iniFile.writeSection("Filter");
				writeFilterSettings(iniFile,servMgr->filters[i]);
			iniFile.writeLine("[End]");
		}


		iniFile.writeSection("Server1");
			writeServerSettings(iniFile,allowServer1);
		iniFile.writeLine("[End]");

		iniFile.writeSection("Server2");
			writeServerSettings(iniFile,allowServer2);
		iniFile.writeLine("[End]");


		iniFile.writeSection("Broadcast");
		iniFile.writeIntValue("broadcastMsgInterval",chanMgr->broadcastMsgInterval);
		iniFile.writeStrValue("broadcastMsg",chanMgr->broadcastMsg.cstr());
		iniFile.writeIntValue("icyMetaInterval",chanMgr->icyMetaInterval);

		char idStr[64];
		chanMgr->broadcastID.toStr(idStr);
		iniFile.writeStrValue("broadcastID",idStr);



		
		iniFile.writeIntValue("deadHitAge",chanMgr->deadHitAge);



		iniFile.writeSection("Debug");
		iniFile.writeBoolValue("logDebug",(showLog&(1<<LogBuffer::T_DEBUG))!=0);
		iniFile.writeBoolValue("logErrors",(showLog&(1<<LogBuffer::T_ERROR))!=0);
		iniFile.writeBoolValue("logNetwork",(showLog&(1<<LogBuffer::T_NETWORK))!=0);
		iniFile.writeBoolValue("logChannel",(showLog&(1<<LogBuffer::T_CHANNEL))!=0);
		iniFile.writeBoolValue("pauseLog",pauseLog);
		iniFile.writeIntValue("idleSleepTime",sys->idleSleepTime);


		for(i=0; i<ChanMgr::MAX_CHANNELS; i++)
		{
			char idstr[64];
			Channel *c = &chanMgr->channels[i];
			if (c->isActive())
				if ( (c->type == Channel::T_RELAY) ||
					 ((c->type == Channel::T_BROADCAST) && (c->srcType == Channel::SRC_URL)))
				{
					c->getIDStr(idstr);
					iniFile.writeSection("RelayChannel");
					iniFile.writeStrValue("name",c->getName());
					iniFile.writeStrValue("genre",c->info.genre.cstr());
					iniFile.writeStrValue("type",c->info.getTypeStr());
					iniFile.writeStrValue("contactURL",c->info.url.cstr());
					iniFile.writeStrValue("id",idstr);
					iniFile.writeBoolValue("stayConnected",c->stayConnected);
					if (!c->sourceURL.isEmpty())
						iniFile.writeStrValue("sourceURL",c->sourceURL.cstr());
					iniFile.writeLine("[End]");
				}
		}


		

		iniFile.writeSection("HostCache");

		char str[64];
		for(i=0; i<ServMgr::MAX_SERVENTS; i++)
			if (servMgr->servents[i].type == Servent::T_OUTGOING)
				if (servMgr->servents[i].status == Servent::S_CONNECTED)
				{
					Host h = servMgr->servents[i].getHost();
					h.toStr(str);
					iniFile.writeLine(str);
				}

		for(i=0; i<ServMgr::MAX_HOSTCACHE; i++)
		{
			if (servMgr->hostCache[i].isValid())
			{
				servMgr->hostCache[i].toStr(str);
				iniFile.writeLine(str);
			}
		}
		iniFile.writeLine("[End]");


		iniFile.close();
	}
}
// --------------------------------------------------
unsigned int readServerSettings(IniFile &iniFile, unsigned int a)
{
	while (iniFile.readNext())
	{
		if (iniFile.isName("[End]"))
			break;
		else if (iniFile.isName("allowHTML"))
			a = iniFile.getBoolValue()?a|Servent::ALLOW_HTML:a&~Servent::ALLOW_HTML;
		else if (iniFile.isName("allowData"))
			a = iniFile.getBoolValue()?a|Servent::ALLOW_DATA:a&~Servent::ALLOW_DATA;
		else if (iniFile.isName("allowServent"))
			a = iniFile.getBoolValue()?a|Servent::ALLOW_SERVENT:a&~Servent::ALLOW_SERVENT;
		else if (iniFile.isName("allowBroadcast"))
			a = iniFile.getBoolValue()?a|Servent::ALLOW_BROADCAST:a&~Servent::ALLOW_BROADCAST;
	}
	return a;
}
// --------------------------------------------------
void readFilterSettings(IniFile &iniFile, ServFilter &sv)
{
	sv.flags = 0;
	sv.host.init();

	while (iniFile.readNext())
	{
		if (iniFile.isName("[End]"))
			break;
		else if (iniFile.isName("ip"))
			sv.host.fromStr(iniFile.getStrValue());
		else if (iniFile.isName("private"))
			sv.flags |= iniFile.getBoolValue()?ServFilter::F_PRIVATE:0;
		else if (iniFile.isName("ban"))
			sv.flags |= iniFile.getBoolValue()?ServFilter::F_BAN:0;
		else if (iniFile.isName("allow"))
			sv.flags |= iniFile.getBoolValue()?ServFilter::F_ALLOW:0;
	}
}
// --------------------------------------------------
void ServMgr::loadSettings(const char *fn)
{
	IniFile iniFile;

	if (!iniFile.openReadOnly(fn))
		saveSettings(fn);


	servMgr->numFilters = 0;
	showLog = 0;

	if (iniFile.openReadOnly(fn))
	{
		while (iniFile.readNext())
		{
			// server settings
			if (iniFile.isName("serverPort"))
				servMgr->serverHost.port = iniFile.getIntValue();
			else if (iniFile.isName("autoServe"))
				servMgr->autoServe = iniFile.getBoolValue();
			else if (iniFile.isName("autoConnect"))
				servMgr->autoConnect = iniFile.getBoolValue();
			else if (iniFile.isName("icyPassword"))
				strcpy(servMgr->password,iniFile.getStrValue());
			else if (iniFile.isName("forceIP"))
			{
				strcpy(servMgr->forceIP,iniFile.getStrValue());
				if (strlen(servMgr->forceIP))
					servMgr->serverHost.ip = ClientSocket::getIP(servMgr->forceIP);
			}
			else if (iniFile.isName("isRoot"))
				servMgr->isRoot = iniFile.getBoolValue();
			else if (iniFile.isName("broadcastID"))
				chanMgr->broadcastID.fromStr(iniFile.getStrValue());
			else if (iniFile.isName("maxBitrate"))
				servMgr->maxBitrate = iniFile.getIntValue();
			else if (iniFile.isName("maxStreams"))
				servMgr->maxStreams = iniFile.getIntValue();
			else if (iniFile.isName("maxStreamsPerChannel"))
				chanMgr->maxStreamsPerChannel = iniFile.getIntValue();
			else if (iniFile.isName("maxIncoming"))
				servMgr->maxIncoming = iniFile.getIntValue();
			else if (iniFile.isName("maxOutgoing"))
				servMgr->maxOutgoing = iniFile.getIntValue();
			else if (iniFile.isName("maxTryout"))
				servMgr->maxTryout = iniFile.getIntValue();
			else if (iniFile.isName("minConnected"))
				servMgr->minConnected = iniFile.getIntValue();
			else if (iniFile.isName("serventBandwidth"))
				servMgr->serventBandwidth = iniFile.getIntValue();
			else if (iniFile.isName("firewallTimeout"))
				firewallTimeout = iniFile.getIntValue();
			else if (iniFile.isName("forceNormal"))
				forceNormal = iniFile.getBoolValue();
			else if (iniFile.isName("broadcastMsgInterval"))
				chanMgr->broadcastMsgInterval = iniFile.getIntValue();
			else if (iniFile.isName("broadcastMsg"))
				chanMgr->broadcastMsg.set(iniFile.getStrValue(),String::T_ASCII);
			else if (iniFile.isName("icyMetaInterval"))
				chanMgr->icyMetaInterval = iniFile.getIntValue();
			else if (iniFile.isName("rootMsg"))
				rootMsg.set(iniFile.getStrValue());
			else if (iniFile.isName("network"))
				network.set(iniFile.getStrValue());
			else if (iniFile.isName("authType"))
			{
				char *t = iniFile.getStrValue();
				if (stricmp(t,"cookie")==0)
					servMgr->authType = ServMgr::AUTH_COOKIE;
				else if (stricmp(t,"http-basic")==0)
					servMgr->authType = ServMgr::AUTH_HTTPBASIC;
			}else if (iniFile.isName("cookiesExpire"))
			{
				char *t = iniFile.getStrValue();
				if (stricmp(t,"never")==0)
					servMgr->cookieList.neverExpire = true;
				else if (stricmp(t,"session")==0)
					servMgr->cookieList.neverExpire = false;

			// client settings
			}else if (iniFile.isName("lookupHost"))
				strcpy(servMgr->connectHost,iniFile.getStrValue());
			else if (iniFile.isName("deadHitAge"))
				chanMgr->deadHitAge = iniFile.getIntValue();
			else if (iniFile.isName("tryoutDelay"))
				servMgr->tryoutDelay = iniFile.getIntValue();
			else if (iniFile.isName("refreshHTML"))
				refreshHTML = iniFile.getIntValue();
			else if (iniFile.isName("relayBroadcast"))
				servMgr->relayBroadcast = iniFile.getIntValue();

			// debug
			else if (iniFile.isName("logDebug"))
				showLog |= iniFile.getBoolValue() ? 1<<LogBuffer::T_DEBUG:0;
			else if (iniFile.isName("logErrors"))
				showLog |= iniFile.getBoolValue() ? 1<<LogBuffer::T_ERROR:0;
			else if (iniFile.isName("logNetwork"))
				showLog |= iniFile.getBoolValue() ? 1<<LogBuffer::T_NETWORK:0;
			else if (iniFile.isName("logChannel"))
				showLog |= iniFile.getBoolValue() ? 1<<LogBuffer::T_CHANNEL:0;
			else if (iniFile.isName("pauseLog"))
				pauseLog = iniFile.getBoolValue();
			else if (iniFile.isName("idleSleepTime"))
				sys->idleSleepTime = iniFile.getIntValue();
			else if (iniFile.isName("[Server1]"))
				allowServer1 = readServerSettings(iniFile,allowServer1);
			else if (iniFile.isName("[Server2]"))
				allowServer2 = readServerSettings(iniFile,allowServer2);
			else if (iniFile.isName("[Filter]"))
			{
				readFilterSettings(iniFile,filters[numFilters]);

				if (numFilters < (MAX_FILTERS-1))
					numFilters++;
			}
			else if (iniFile.isName("[Notify]"))
			{
				sys->popupNotifyMask = Sys::NT_UPGRADE;
				while (iniFile.readNext())
				{
					if (iniFile.isName("[End]"))
						break;
					else if (iniFile.isName("PeerCast"))
						sys->popupNotifyMask |= iniFile.getBoolValue()?Sys::NT_PEERCAST:0;
					else if (iniFile.isName("Broadcasters"))
						sys->popupNotifyMask |= iniFile.getBoolValue()?Sys::NT_BROADCASTERS:0;
					else if (iniFile.isName("TrackInfo"))
						sys->popupNotifyMask |= iniFile.getBoolValue()?Sys::NT_TRACKINFO:0;
				}

			}
			else if (iniFile.isName("[RelayChannel]"))
			{
				ChanInfo info;
				bool stayConnected=false;
				String sourceURL;
				while (iniFile.readNext())
				{
					if (iniFile.isName("[End]"))
						break;
					else if (iniFile.isName("name"))
						info.name.set(iniFile.getStrValue());
					else if (iniFile.isName("id"))
						info.id.fromStr(iniFile.getStrValue());
					else if (iniFile.isName("type"))
						info.type = ChanInfo::getTypeFromStr(iniFile.getStrValue());
					else if (iniFile.isName("stayConnected"))
						stayConnected = iniFile.getBoolValue();
					else if (iniFile.isName("sourceURL"))
						sourceURL.set(iniFile.getStrValue());
					else if (iniFile.isName("genre"))
						info.genre.set(iniFile.getStrValue());
					else if (iniFile.isName("contactURL"))
						info.url.set(iniFile.getStrValue());

				}
				if (sourceURL.isEmpty())
				{
					chanMgr->createRelay(info,stayConnected);
				}else
				{
					Channel *c = chanMgr->createChannel(info,NULL);
					if (c)
						c->startURL(sourceURL.cstr());
				}
			}


			// host cache
			else if (iniFile.isName("[HostCache]"))
			{
				while (iniFile.readNext())
				{
					if (iniFile.isName("[End]"))
						break;
					Host h;
					h.fromStr(iniFile.getName());
					if (h.isValid())
					{
						char tt[64];
						h.toStr(tt);
						servMgr->addHost(h);
					}
					
				}

				// reset read ptr to get last connected nodes first
				servMgr->readHostPtr = 0;
			}


		}
	}

	if (!numFilters)
		setFilterDefaults();

}
// --------------------------------------------------
int ServMgr::findChannel(ChanInfo &info)
{
	LOG_NETWORK("Channel search: %s",info.name.cstr());
	GnuPacket pack;
	pack.initFind(info.name,NULL);
	addReplyID(pack.id);
	return broadcast(pack,NULL);
}
// --------------------------------------------------
bool ServMgr::start()
{
	static ThreadInfo serverThread,clientThread,idleThread;

	serverThread.func = ServMgr::serverProc;
	if (!sys->startThread(&serverThread))
		return false;

	clientThread.func = ServMgr::clientProc;
	if (!sys->startThread(&clientThread))
		return false;

	idleThread.func = ServMgr::idleProc;
	if (!sys->startThread(&idleThread))
		return false;

	return true;
}
// --------------------------------------------------
int ServMgr::clientProc(ThreadInfo *thread)
{

	thread->lock();

	while(thread->active)
	{
		if (servMgr->autoConnect)
		{
			if ((servMgr->needConnections() && !servMgr->outFull()) || servMgr->forceLookup )
			{
				if (servMgr->needHosts() || servMgr->forceLookup)
				{
					char host[128];
					strcpy(host,servMgr->connectHost);
					int port = 7144;
					char *pp = strstr(host,":");
					if (pp)
					{
						port = atoi(pp+1);
						pp[0] = 0;
					}

					int ip = ClientSocket::getIP(host);

					if (!servMgr->findServent(ip,port))
					{
						Servent *sv = servMgr->allocServent();
						if (sv)
						{
							LOG_DEBUG("Lookup: %s:%d",host,port);
							sv->initOutgoing(host,port,Servent::T_LOOKUP);
							servMgr->forceLookup = false;
						}
					}
				}

				for(int i=0; i<MAX_TRYOUT; i++)
				{
					if (servMgr->outFull())
						break;
					if (servMgr->tryFull())
						break;

					Host h = servMgr->getHost();

					Host lh(ClientSocket::getIP(NULL),0);
					if (lh.globalIP() && !h.globalIP())
						h.ip = 0;

					if (h.ip) 
						if (!servMgr->findServent(h.ip,h.port))
						{
							char hostName[64];
							h.IPtoStr(hostName);
							Servent *sv = servMgr->allocServent();
							if (sv)
								sv->initOutgoing(hostName,h.port,Servent::T_OUTGOING);
							else
								break;
						}
					sys->sleep(servMgr->tryoutDelay);
				}
			}
		}else{
			for(int i=0; i<ServMgr::MAX_SERVENTS; i++)
			{
				Servent *s = &servMgr->servents[i];
				if (s->type == Servent::T_OUTGOING) 
					s->thread.active = false;

			}
		}
		sys->sleepIdle();
	}
	thread->unlock();
	return 0;
}
// --------------------------------------------------
int ServMgr::idleProc(ThreadInfo *thread)
{

	thread->lock();

	unsigned int lastPasvFind=0;
	unsigned int lastBroadcast=0;


	// nothing much to do for the first couple of seconds, so just hang around.
	sys->sleep(2000);	

	unsigned int lastBWcheck=0;
	unsigned int bytesIn=0,bytesOut=0;

	while(thread->active)
	{
		stats.update();


		if (servMgr->relayBroadcast)
			if ((sys->getTime()-lastBroadcast) > servMgr->relayBroadcast)
			{
				chanMgr->broadcastRelays(7,NULL);
				lastBroadcast = sys->getTime();
			}

#if 0
		if (servMgr->passiveSearch)
			if ((sys->getTime()-lastPasvFind) > servMgr->passiveSearch)
			{
				LOG_DEBUG("Passive find");
				GnuPacket pack;
				pack.initFind("",NULL);
				servMgr->addActiveID(pack.id);
				servMgr->broadcast(pack,NULL);
				lastPasvFind = sys->getTime();
			}
#endif

#if 0
		if (chanMgr->searchActive)
		{
			if ((sys->getTime()-chanMgr->lastSearch) > 15)		
			{
				if (chanMgr->numFinds >= 2)
					chanMgr->searchActive = false;
				else
				{
					if (servMgr->findChannel(chanMgr->searchInfo))
					{
						LOG_DEBUG("Auto find: %d",chanMgr->numFinds);
						chanMgr->lastSearch = sys->getTime();
						chanMgr->numFinds++;
					}
				}
			}
		}
#endif
		// clear dead hits

		chanMgr->clearDeadHits();

		// kill old idle channels
		{
			int cnt=0;
			Channel *oldest=NULL;
			unsigned int last = (unsigned int )-1;
			for(int i=0; i<ChanMgr::MAX_CHANNELS; i++)
			{
				Channel *c = &chanMgr->channels[i];
				if (c->isIdle())
				{
					cnt++;
					if (c->lastIdleTime < last)
					{
						oldest = c;
						last = c->lastIdleTime;
					}
				}
			}
			if (cnt > ChanMgr::MAX_IDLE)
				if (oldest)
					oldest->thread.active = false;
		}


		//if (!servMgr->isRelay)
		{
			// check for too many connections 
			bool killIn = servMgr->inOver();
			bool killOut = servMgr->outOver();
			for(int i=0; i<ServMgr::MAX_SERVENTS; i++)
			{
				Servent *s = &servMgr->servents[i];
				if (s->status == Servent::S_CONNECTED)
				{
					if ( (killOut && (s->type == Servent::T_OUTGOING)) ||
						 (killIn && (s->type == Servent::T_INCOMING)) )
					{
						s->thread.active = false;
						break;
					}
				}

			}
		}

		if (servMgr->shutdownTimer)
		{
			if (--servMgr->shutdownTimer <= 0)
			{
				debugtest = 1;
				servMgr->saveSettings(iniFileName.cstr());
				sys->exit();
			}
		}



		sys->sleep(500);
	}

	thread->unlock();
	return 0;
}

// --------------------------------------------------
int ServMgr::serverProc(ThreadInfo *thread)
{
	thread->lock();

	Servent *serv = servMgr->allocServent();
	serv->permAlloc = true;

	Servent *serv2 = servMgr->allocServent();
	serv2->permAlloc = true;

	unsigned int lastLookupTime=0;


	while (thread->active)
	{
		if (servMgr->restartServer)
		{
			serv->abort();		// force close

			serv2->abort();		// force close

			servMgr->restartServer = false;
		}

		if (servMgr->autoServe)
		{
			serv->allow = servMgr->allowServer1;
			serv2->allow = servMgr->allowServer2;

			if ((serv->lastConnect) && ((sys->getTime()-serv->lastConnect) > servMgr->firewallTimeout))
			{
				if (servMgr->getFirewall()==ServMgr::FW_UNKNOWN)
				{
					LOG_DEBUG("Server has had no incoming connections");
					servMgr->setFirewall(ServMgr::FW_ON);
				}
			}


			// force lookup every 24 hours to check for new messages
			if ((sys->getTime()-lastLookupTime) > 60*60*24)
			{
				servMgr->forceLookup = true;
				lastLookupTime = sys->getTime();
			}


			if ((serv->status == Servent::S_NONE) || (serv2->status == Servent::S_NONE))
			{
				LOG_DEBUG("Starting servers");
				servMgr->forceLookup = true;

				//if (servMgr->serverHost.ip != 0)
				{

					if (servMgr->forceNormal)
						servMgr->setFirewall(ServMgr::FW_OFF);
					else
						servMgr->setFirewall(ServMgr::FW_UNKNOWN);

					Host h = servMgr->serverHost;

					if (serv->status == Servent::S_NONE)
						serv->initServer(h);

					h.port++;
					if (serv2->status == Servent::S_NONE)
						serv2->initServer(h);


				}
			}
		}else{
			// stop server
			serv->abort();		// force close
			serv2->abort();		// force close

			// cancel incoming connectuions
			for(int i=0; i<ServMgr::MAX_SERVENTS; i++)
			{
				Servent *s = &servMgr->servents[i];
				if (s->type == Servent::T_INCOMING)
					s->thread.active = false;
			}

			servMgr->setFirewall(ServMgr::FW_ON);
		}
			
		sys->sleepIdle();

	}

	thread->unlock();
	return 0;
}

