// ------------------------------------------------
// File : lsys.cpp
// Date: 4-apr-2002
// Author: giles
// Desc: 
//		LSys derives from Sys to provide basic Linux functions such as starting threads.
//
// (c) 2002 peercast.org
// ------------------------------------------------
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------


#include <time.h>
#ifdef BEOS
# include <kernel/OS.h>
#else
# include <pthread.h>
#endif
#include <signal.h>
#include "lsys.h"
#include "lsocket.h"
#include "stats.h"

// ---------------------------------
LSys::LSys()
{
	stats.clear();

	rndGen.setSeed(getTime());
	signal(SIGPIPE, SIG_IGN); 

	rndSeed = rnd();

}
// ---------------------------------
unsigned int LSys::getTime()
{
	time_t ltime;
	time( &ltime );
	return ltime;
}

// ---------------------------------
ClientSocket *LSys::createSocket()
{
    return new LClientSocket();
}
               
typedef void *(*THREAD_PTR)(void *);

// ---------------------------------
bool	LSys::startThread(ThreadInfo *info)
{
	info->active = true;

#ifdef BEOS
	thread_id thread = spawn_thread((thread_func)info->func, "LSys thread", B_NORMAL_PRIORITY, info);
	if (thread >= 0)
	{
		resume_thread(thread);
		return true;
	}
	else return false;	
#else
	pthread_t thread;
	pthread_attr_t attr;

	pthread_attr_init(&attr);
	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

	return pthread_create(&thread,&attr,(THREAD_PTR)info->func,info) == 0;
#endif
}
// ---------------------------------
void	LSys::sleep(int ms)
{
	::usleep(ms*1000);
}

// ---------------------------------
void LSys::appMsg(long msg, long arg)
{
	//SendMessage(mainWindow,WM_USER,(WPARAM)msg,(LPARAM)arg);
}

// ---------------------------------
void LSys::getURL(const char *url)
{
}
// ---------------------------------
void LSys::exit()
{
	::exit(0);
}

