// ------------------------------------------------
// File : lsocket.cpp
// Date: 4-apr-2002
// Author: giles
// Desc: 
//		Linux version of ClientSocket. Handles the nitty gritty of actually
//		reading and writing TCP
//		
// (c) 2002 peercast.org
// ------------------------------------------------


#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>
#include "xsocket.h"
#include "stats.h"

#define INVALID_SOCKET -1
#define SOCKET_ERROR -1

// --------------------------------------------------
void XClientSocket::init()
{
    LOG_DEBUG("MStartup:  OK");

}

// --------------------------------------------------
bool ClientSocket::getHostname(char *str,unsigned int ip)
{
	hostent *he;

	ip = htonl(ip);

	he = gethostbyaddr((char *)&ip,sizeof(ip),AF_INET);

	if (he)
	{
		strcpy(str,he->h_name);
		return true;
	}else
		return false;
}
// --------------------------------------------------
unsigned int ClientSocket::getIP(char *name)
{

	char szHostName[256];

	if (!name)
	{
		if (gethostname(szHostName, sizeof(szHostName))==0)
			name = szHostName;
		else
			return 0;
	}

	hostent *he = XClientSocket::resolveHost(name);

	if (!he)
		return 0;


	char* lpAddr = he->h_addr_list[0];
	if (lpAddr)
	{
		struct in_addr  inAddr;
		memmove (&inAddr, lpAddr, 4);

		return ntohl(inAddr.s_addr);
	}
	return 0;
}
// --------------------------------------------------
void XClientSocket::setBlocking(bool yes)
{
	if (!yes)
		fcntl(sockNum, F_SETFL, O_NONBLOCK);
}

// --------------------------------------------------
void XClientSocket::setReuse(bool yes)
{

	unsigned long op = yes ? 1 : 0;
	if (setsockopt(sockNum,SOL_SOCKET,SO_REUSEADDR,(char *)&op,sizeof(op)) < 0) 
		LOG_ERROR("Unable to set REUSE");
}

// --------------------------------------------------
hostent *XClientSocket::resolveHost(char *hostName)
{
	hostent *he;

	if ((he = gethostbyname(hostName)) == NULL)
	{
		// if failed, try using gethostbyaddr instead

		unsigned long ip = inet_addr(hostName);

		ip = htonl( ip );

		if (ip == INADDR_NONE)
			return NULL;

		if ((he = gethostbyaddr((char *)&ip,sizeof(ip),AF_INET)) == NULL)
			return NULL;	
	}
	return he;
}

// --------------------------------------------------
void XClientSocket::open(char *hostName, int port)
{

//	LOG("Open: %s:%d",hostName,port);

	hostent *he = resolveHost(hostName);

	if (!he)
		throw SockException("Unable to resolve host");

	sockNum = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);

	if (sockNum == INVALID_SOCKET)
		throw SockException("Can`t open socket");

	setBlocking(false);

	//setReuse(true);

//	int on = 1;
//	if (setsockopt(sockNum, SOL_SOCKET, TCP_NODELAY, (char *)&on, sizeof on) == -1) 
//		LOG("Unable to set NODELAY");

	memset(&remoteAddr,0,sizeof(remoteAddr));

	remoteAddr.sin_family = AF_INET;
	remoteAddr.sin_port = htons(port);
	remoteAddr.sin_addr.s_addr = ((struct in_addr *)(he->h_addr))->s_addr;

	host.port = port;
	host.ip = remoteAddr.sin_addr.s_addr;
}

// --------------------------------------------------
void XClientSocket::checkTimeout(bool r, bool w)
{
    int err = errno;
    if ((err == EAGAIN) || (err == EINPROGRESS))
    {
		//LOG("checktimeout %d %d",(int)r,(int)w);

		timeval timeout;
		fd_set read_fds;
		fd_set write_fds;

		timeout.tv_sec = (int)this->timeout/1000;
		timeout.tv_usec = 0;

        FD_ZERO (&write_fds);
		if (w)
			FD_SET (sockNum, &write_fds);

        FD_ZERO (&read_fds);
		if (r)
	        FD_SET (sockNum, &read_fds);

		int r=select (sockNum+1, &read_fds, &write_fds, NULL, &timeout);

        if (r == 0)
            throw TimeoutException();
		else if (r == SOCKET_ERROR)
			throw SockException("select failed.");
	}else{
		char str[32];
		sprintf(str,"Closed: %s",sys_errlist[err]);
		throw SockException(str);
	}

}

// --------------------------------------------------
void XClientSocket::connect()
{
	if (::connect(sockNum,(struct sockaddr *)&remoteAddr,sizeof(remoteAddr)) == SOCKET_ERROR)
		checkTimeout(false,true);
}
// --------------------------------------------------
int XClientSocket::read(void *p, int l)
{
	int bytesRead=0;
	while (l)
	{
//		int r = recv(sockNum, (char *)p, l, 0);
        int r = ::read(sockNum, (char *)p, l );
		if (r == SOCKET_ERROR)
		{
			checkTimeout(true,false);		// non-blocking sockets always fall through to here
		}else if (r == 0)
		{
			throw SockException("Closed on read");
		}else
		{
			stats.add(Stats::BYTESIN,r);
			if (host.localIP())
				stats.add(Stats::LOCALBYTESIN,r);
			updateTotals(r,0);
			bytesRead+=r;
			l -= r;
			p = (char *)p+r;
		}
	}

	return bytesRead;
}


// --------------------------------------------------
void XClientSocket::write(const void *p, int l)
{
	while (l)
	{
		int r = send(sockNum, (char *)p, l, MSG_DONTWAIT );
		if (r == SOCKET_ERROR)
		{
			checkTimeout(false,true);		// non-blocking sockets always fall through to here
		}else if (r == 0)
		{
			throw SockException("Closed on write");
		}else
		{
			stats.add(Stats::BYTESOUT,r);
			if (host.localIP())
				stats.add(Stats::LOCALBYTESOUT,r);
			updateTotals(0,r);
			l -= r;
			p = (char *)p+r;
		}
	}
}


// --------------------------------------------------
void XClientSocket::bind(Host &h)
{
	struct sockaddr_in localAddr;

	if ((sockNum = socket (PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1)
		throw SockException("Can`t open socket");

	setReuse(true);

	memset(&localAddr,0,sizeof(localAddr));
	localAddr.sin_family = AF_INET;
	localAddr.sin_port = htons(h.port);
	localAddr.sin_addr.s_addr = INADDR_ANY;

	if( ::bind (sockNum, (sockaddr *)&localAddr, sizeof(localAddr)) == -1)
		throw SockException("Can`t bind socket");

	if (::listen(sockNum,SOMAXCONN))
		throw SockException("Can`t listen");

	host = h;
}
// --------------------------------------------------
ClientSocket *XClientSocket::accept()
{

	socklen_t fromSize = sizeof(sockaddr_in);
	sockaddr_in from;

	int conSock = ::accept(sockNum,(sockaddr *)&from,&fromSize);


	if (conSock ==  INVALID_SOCKET)
		return NULL;

	
    XClientSocket *cs = new XClientSocket();
	cs->sockNum = conSock;

	cs->host.port = from.sin_port;
	cs->host.ip = ntohl(from.sin_addr.s_addr);

	cs->setBlocking(false);

	return cs;
}

// --------------------------------------------------
Host XClientSocket::getLocalHost()
{
	struct sockaddr_in localAddr;

	socklen_t len = sizeof(localAddr);
    if (getsockname(sockNum, (sockaddr *)&localAddr, &len) == 0)
		return Host(ntohl(localAddr.sin_addr.s_addr),0);
	else
		return Host(0,0);
}
// --------------------------------------------------
void XClientSocket::close()
{
	if (sockNum)
	{
		::close(sockNum);

		sockNum = 0;
	}
}

// --------------------------------------------------
int XClientSocket::readPending() 
{
	size_t len;

    if (ioctl( sockNum, FIONREAD, (char *)&len ) < 0)
		throw StreamException("readPending");

	return (int)len;
}

